searchData={"content_type":"text/markdown","items":[{"doc":"Declarative programming in Elixir\n\nDiesel is a toolkit that helps you build your own DSLs.\n\nUsage:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel,\n    otp_app: :my_app,\n    dsl: MyApp.Fsm.Dsl,\n    parsers: [\n      ...\n    ],\n    generators: [\n      ...\n    ]\nend\n```\n\n\nFor more information on how to use this library, please check:\n\n* the `Diesel.Dsl` and `Diesel.Tag` modules,\n* the guides and tutorials provided in the documentation\n* the examples used in tests","ref":"Diesel.html","title":"Diesel","type":"behaviour"},{"doc":"Returns the raw definition for the dsl, before compilation","ref":"Diesel.html#c:definition/0","title":"Diesel.definition/0","type":"callback"},{"doc":"","ref":"Diesel.html#t:element/0","title":"Diesel.element/0","type":"type"},{"doc":"","ref":"Diesel.html#t:tag/0","title":"Diesel.tag/0","type":"type"},{"doc":"Defines the syntax provided by a DSL.\n\nSimple usage:\n\n```elixir\ndefmodule MyApp.Fsm.Dsl do\n  use Diesel.Dsl,\n    root: :fsm,\n    tags: [\n      :state,\n      :on,\n      :action,\n      :next\n    ]\nend\n```\n\nPlease check the documentation for more info on how to extend DSLs via `packages`","ref":"Diesel.Dsl.html","title":"Diesel.Dsl","type":"module"},{"doc":"Generators produce actual Elixir code from DSLs.\n\nGiven a definition, a generator returns one or multiple quoted expressions.\n\nThe following info are passed in the keyword list of options:\n\n- `:otp_app`\n- `:caller_module`","ref":"Diesel.Generator.html","title":"Diesel.Generator","type":"behaviour"},{"doc":"","ref":"Diesel.Generator.html#c:generate/2","title":"Diesel.Generator.generate/2","type":"callback"},{"doc":"Naming utilities","ref":"Diesel.Naming.html","title":"Diesel.Naming","type":"module"},{"doc":"Return a module name","ref":"Diesel.Naming.html#module_name/1","title":"Diesel.Naming.module_name/1","type":"function"},{"doc":"A parser is a DSL tranformation step before code generation\n\nExample:\n\n```elixir\ndef parse({:music, attrs, children}, _opts) do\n  # eg return some struct here\nend\n```\n\nParsing a definition is an optional step.","ref":"Diesel.Parser.html","title":"Diesel.Parser","type":"behaviour"},{"doc":"Returns a built-in parser, given its name\n\nSupported names:\n\n* `strip_root`","ref":"Diesel.Parser.html#named/1","title":"Diesel.Parser.named/1","type":"function"},{"doc":"","ref":"Diesel.Parser.html#c:parse/2","title":"Diesel.Parser.parse/2","type":"callback"},{"doc":"Tags are the building blocks for the syntax offered by a Diesel DSL.\n\nUsage:\n\n```elixir\ndefmodule MyApp.Fsm.Dsl.State do\n  use Diesel.Tag\n\n  attribute :name, kind: :atom\n  child :on, :min: 0\nend\n```\n\nFor more information on how to define structured tags, please check the examples provided in the documentation and in tests","ref":"Diesel.Tag.html","title":"Diesel.Tag","type":"module"},{"doc":"","ref":"Diesel.Tag.html#dsl/0","title":"Diesel.Tag.dsl/0","type":"function"},{"doc":"Returns the name of the given tag\n\nFor unstructured, plain atom tags, the name is simply the atom itself","ref":"Diesel.Tag.html#name/1","title":"Diesel.Tag.name/1","type":"function"},{"doc":"","ref":"Diesel.Tag.html#parsers/0","title":"Diesel.Tag.parsers/0","type":"function"},{"doc":"Returns whether a tag is structured or not","ref":"Diesel.Tag.html#structured?/1","title":"Diesel.Tag.structured?/1","type":"function"},{"doc":"Validates the given definition node against the given tag\n\nThis function supports both structured and plain atom tags.\n\nFor plain atom tags, this function will always return success.","ref":"Diesel.Tag.html#validate/2","title":"Diesel.Tag.validate/2","type":"function"},{"doc":"The meta-dsl that allow developers to design structured tags","ref":"Diesel.Tag.Dsl.html","title":"Diesel.Tag.Dsl","type":"module"},{"doc":"","ref":"Diesel.Tag.Dsl.html#attribute/0","title":"Diesel.Tag.Dsl.attribute/0","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#attribute/1","title":"Diesel.Tag.Dsl.attribute/1","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#attribute/2","title":"Diesel.Tag.Dsl.attribute/2","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#attribute/3","title":"Diesel.Tag.Dsl.attribute/3","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#child/0","title":"Diesel.Tag.Dsl.child/0","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#child/1","title":"Diesel.Tag.Dsl.child/1","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#child/2","title":"Diesel.Tag.Dsl.child/2","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#child/3","title":"Diesel.Tag.Dsl.child/3","type":"macro"},{"doc":"Returns the list of locals without parens function signatures, so that they can be easily\nincluded in .formatter.exs files","ref":"Diesel.Tag.Dsl.html#locals_without_parens/0","title":"Diesel.Tag.Dsl.locals_without_parens/0","type":"function"},{"doc":"","ref":"Diesel.Tag.Dsl.html#root/0","title":"Diesel.Tag.Dsl.root/0","type":"function"},{"doc":"","ref":"Diesel.Tag.Dsl.html#tag/1","title":"Diesel.Tag.Dsl.tag/1","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#tag/2","title":"Diesel.Tag.Dsl.tag/2","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#tag/3","title":"Diesel.Tag.Dsl.tag/3","type":"macro"},{"doc":"","ref":"Diesel.Tag.Dsl.html#tags/0","title":"Diesel.Tag.Dsl.tags/0","type":"function"},{"doc":"","ref":"Diesel.Tag.Dsl.html#validate/1","title":"Diesel.Tag.Dsl.validate/1","type":"function"},{"doc":"Helper module that validates a tag against a schema.\n\nValidation of tags is only available for structured tags, and is applied right after tag parsing\nand before code generation.","ref":"Diesel.Tag.Validator.html","title":"Diesel.Tag.Validator","type":"module"},{"doc":"Validates the given node, against the given schema","ref":"Diesel.Tag.Validator.html#validate/2","title":"Diesel.Tag.Validator.validate/2","type":"function"},{"doc":"# Overview\n\n**Diesel** is a toolkit that allows you to design DSLs in Elixir.\n\nDSLs built with Diesel are:\n\n* **declarative**: they look just like HTML\n* **structured**: the schema is checked at compile time\n* **reusable**: the same dsl can be applied to different sets of code generators","ref":"overview.html","title":"Overview","type":"extras"},{"doc":"# Installation\n\nThe package can be installed by adding `diesel` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:diesel, \"~> 0.8\"}\n  ]\nend\n```\n\n# Formatter setup\n\nIn order to avoid superfluous parenthesis when defining structured tags in your DSL, also add `diesel` to your dependencies in your `.formatter.exs`:\n\n```elixir\n[\n  inputs: [ ... ],\n  import_deps: [..., :diesel],\n  locals_without_parens: ...,\n]\n```","ref":"installation.html","title":"Installation","type":"extras"},{"doc":"# Tutorial\n\nIn this tutorial, we will go through the process of designing a DSL for a finite state machine. We will\nuse such state machine to express the lifecycle of a payment.","ref":"tutorial.html","title":"Tutorial","type":"extras"},{"doc":"```elixir\ndefmodule MyApp.Payments.Payment do\n  use MyApp.Fsm\n\n  fsm do\n    state :pending do\n      on event: :created do\n        action SendToGateway\n        next state: :sent\n      end\n    end\n\n    state :sent, timeout: 60 do\n      on event: :success do\n        action NotifyParties\n        next state: :accepted\n      end\n\n      on event: :error do\n        action NotifyParties\n        next state: :declined\n      end\n\n      on event: :timeout do\n        action NotifyParties\n        next state: :declined\n      end\n    end\n\n    state :accepted do\n    end\n\n    state :declined do\n    end\n  end\nend\n```","ref":"tutorial.html#a-state-machine-for-payments","title":"A state machine for payments - Tutorial","type":"extras"},{"doc":"First, lets define the `MyApp.Fsm` library module:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel,\n    otp_app: :my_app,\n    dsl: MyApp.Dsm.Dsl  # optional\nend\n```\n\nThis module will import the api offered by the actual dsl, to be implemented in this example by module `MyApp.Fsm.Dsl`.\n\nThe `:dsl` key is optional. If omitted, it will default to the caller module, suffixed by `Dsl`. The above example is equivalent to:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel, otp_app: :my_app\nend\n```","ref":"tutorial.html#the-fsm-library-module","title":"The Fsm library module - Tutorial","type":"extras"},{"doc":"We will need the following elements of the language:\n\n* `fsm`: the root tag of the dsl\n* `state`: a definition of a state\n* `action`: an action to be triggered as soon as we enter a state\n* `on`: the definition of an event, in a given state\n* `next`: the next state to transition into\n\n\n```elixir\ndefmodule MyApp.Fsm.Dsl do\n  use Diesel.Dsl,\n    otp_app: :my_app,\n    root: MyApp.Fsm.Dsl.Fsm, # optional\n    tags: [\n      MyApp.Fsm.Dsl.Action,\n      MyApp.Fsm.Dsl.Next,\n      MyApp.Fsm.Dsl.On,\n      MyApp.Fsm.Dsl.State\n    ]\nend\n```\n\nThe `:root` key is optional. If omitted, a naming convention will be applied, so that the\nabove example is equivalent to:\n\n```elixir\ndefmodule MyApp.Fsm.Dsl do\n  use Diesel.Dsl,\n    otp_app: :my_app,\n    tags: [\n      ...\n    ]\nend\n```\n\nIn the next sections, we will define these as **structured tags** by relying on the `Diesel.Tag` built-in dsl.\n\n##","ref":"tutorial.html#defining-the-dsl","title":"Defining the DSL - Tutorial","type":"extras"},{"doc":"The `fsm` tag is the root of our DSL. It supports one or many `state` children tags:\n\n```elixir\ndefmodule MyApp.Fsm.Dsl.Fsm do\n  use Diesel.Tag\n\n  tag do\n    child :state, min: 1\n  end\nend\n```\n\n##","ref":"tutorial.html#the-fsm-root-tag","title":"The fsm root tag - Tutorial","type":"extras"},{"doc":"The `state` tag requires:\n\n* a `name` attribute (this is the default attribute name in Diesel). The accepted values are:\n`pending`, `sent`, `accepted`, `declined`.\n* an optional `timeout` attribute\n* zero, one or multiple `on` children\n\n```elixir\ndefmodule MyApp.Fsm.Dsl.State do\n  use Diesel.Tag\n\n  tag do\n    attribute :name, kind: :atom, one_of: [:pending, :sent, :accepted, :declined]\n    attribute :timeout, kind: :number, required: false\n    child :on, min: 0\n  end\nend\n```\n\nThe `:name` attribute of any given tag is implicit. For example, the following notation:\n\n```elixir\nstate :pending do\n  ...\nend\n```\n\nis equivalent to:\n\n```elixir\nstate name: :pending do\n  ...\nend\n```\n\n**Note**: since 0.8.0, Diesel supports generic attributes, where the name or the kind are not predefined. This can be set by using the `:*` notation:\n\n```elixir\ndefmodule MyApp.MyDsl.MyTag do\n  use Diesel.Tag\n\n  tag do\n    attribute name: :*, kind: :*, min: 1\n  end\nend\n```\n\n\n##","ref":"tutorial.html#the-state-tag","title":"The state tag - Tutorial","type":"extras"},{"doc":"The `action` supports the name of an Elixir module as its only child:\n\n```elixir\ndefmodule MyApp.Fsm.Dsl.Action do\n  use Diesel.Tag\n\n  tag do\n    child kind: :module, min: 1, max: 1\n  end\nend\n```\n\nWhen there is a single child involved, the notation\n\n```elixir\naction SomeModule\n```\n\nis equivalent to:\n\n```elixir\naction do\n  SomeModule\nend\n```\n\n##","ref":"tutorial.html#the-action-tag","title":"The action tag - Tutorial","type":"extras"},{"doc":"The `on` tag supports:\n\n* the `name` of the event, as an attribute\n* exactly one `next` child, as the next state\n* zero, one or multiple `action` modules to execute as part of the state transition\n\n\n```elixir\ndefmodule MyApp.Fsm.Dsl.On do\n  use Diesel.Tag\n\n  tag do\n    attribute :event, kind: :atom\n    child :next, min: 0, max: 1\n    child :action, min: 0\n  end\nend\n```\n\n##","ref":"tutorial.html#the-on-tag","title":"The on tag - Tutorial","type":"extras"},{"doc":"The `next` tag only supports the next state to transition to, as the `name` attribute:\n\n```elixir\ndefmodule MyApp.Fsm.Dsl.Next do\n  use Diesel.Tag\n\n  tag do\n    attribute :state, kind: :atom, one_of: [:pending, :sent, :accepted, :declined]\n  end\nend\n```","ref":"tutorial.html#the-next-tag","title":"The next tag - Tutorial","type":"extras"},{"doc":"We can convert the resulting dsl into an alternative representation, for easier consumption during code generation.\n\nFor example, if we define a custom `Transition` struct to model state transitions:\n\n```elixir\ndefmodule MyApp.Fsm.Transition do\n  @moduledoc \"\"\"\n  A custom representation of a state transition\n\n  * `from` is the source state\n  * `to` is the target state\n  * `event` is the triggering event\n  * `actions` is a list of Elixir modules to execute as side effects\n  \"\"\"\n  defstruct [:from, :to, :event, :actions]\n```\n\nthen we can define a parser module that transforms the original raw definition (a tree of tuples) into a plain list of `Transition` structrs:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel,\n    otp_app: ...,\n    dsl: ...,\n    parsers: [\n      MyApp.Fsm.Parser\n    ]\nend\n```\n\nPlease check the `Fsm.Parser` module included in `test/support/fsm.ex`.\n\nThe `:parsers` key is optional. If omitted, a default parser will be used, by appending\nthe `Parser` suffix to the caller module. The above example is equivalent to:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel,\n    otp_app: ...,\n    dsl: ...,\nend\n```\n\nIn reality, parsers are optional. If you wish to skip them entirely, you can set an empty list:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel,\n    otp_app: ...,\n    dsl: ...,\n    parsers: []\nend\n```","ref":"tutorial.html#parsing-the-dsl","title":"Parsing the DSL - Tutorial","type":"extras"},{"doc":"Once our state machine is parsed into a list of transitions, we can then generate any custom code of our choice and inject it into our `MyApp.Fsm` module.\n\nGenerated code is provided by implementations of the `Diesel.Generator` behaviour. A generator returns one or more Elixir quoted expressions from its `generate/2` callback.\n\nFor example, in order to generate a `diagram/0` function that returns a Graphviz diagram for our state machine, we could make use of module `Fsm.Diagram`, also included in `test/support/fsm.ex`:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel,\n    otp_app: ...,\n    dsl: ...,\n    parsers: ...,\n    generators: [\n      Fsm.Diagram\n    ]\nend\n```","ref":"tutorial.html#generating-code","title":"Generating code - Tutorial","type":"extras"},{"doc":"# Parsers","ref":"parsers.html","title":"Parsers","type":"extras"},{"doc":"By default, DSLs generated with Diesel are compiled into a tree-like structure of nodes as tuples.\n\nThis is similar to the structure you'd expect from a HTML document parsed with Loki. For example,\nthe following code:\n\n```elixir\ndefmodule MyApp.Fsm.Paymnent do\n  use MyApp.Fsm\n\n  fsm do\n    state :pending do\n    end\n  end\nend\n```\n\ngives the following internal raw definition:\n\n```elixir\n{:fsm, [], [\n  {:state, [name: :pending], []}\n]}\n```\n\nBy default, this is the datastructure that will be then consumed by code generators.","ref":"parsers.html#raw-definitions","title":"Raw definitions - Parsers","type":"extras"},{"doc":"It is possible however to add parsing steps and convert the raw definition into a more suitable data\nstructure, before it is consumed by generators.\n\nAll you need to do is implement the `Diesel.Parser` behaviour in a new elixir module:\n\n```elixir\ndefmodule MyApp.Fsm.MyParser do\n  @moduledoc \"Converts a fsm definition into structs\"\n  @behaviour Diesel.Parser\n\n  @impl true\n  def parse({:fsm, [], states}, _opts) do\n    %Fsm{states: [...]}\n  end\nend\n```\n\nand add it to the list of parsers:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel,\n    otp_app: ...,\n    dsl: ...,\n    parsers: [\n      MyApp.Fsm.MyParser\n    ]\n\n  defstruct states: []\nend\n```","ref":"parsers.html#the-parser-behaviour","title":"The Parser behaviour - Parsers","type":"extras"},{"doc":"# Generators\n\nBy implementing the `Diesel.Generator` behaviour, developers can produce elixir code, based on a DSL\ndefinition. For example, we could imagine a GenServer based implementation of a state machine:\n\n```elixir\ndefmodule MyApp.Fsm do\n  use Diesel,\n    otp_app: ...,\n    dsl: ...,\n    parsers: [...],\n    generators: [\n      MyApp.Fsm.Generator.GenServer\n    ]\n```\n\nwhere:\n\n```elixir\ndefmodule MyApp.Fsm.Generator.GenServer do\n  @moduledoc \"Generates a GenServer for a state machine DSL\"\n  @behaviour Diesel.Generator\n\n  @impl true\n  def generate(definition, _opts) do\n    quote do\n      use GenServer\n      ...\n    end\n  end\nend\n```","ref":"generators.html","title":"Generators","type":"extras"},{"doc":"# Unstructured tags\n\nIn general, it is highly recommended to express DSLs in terms of structured tags, by using the\n`Diesel.Tag` macro. This will enforce a well defined schema and the compiler will be able to pick up\nmisconfigurations and/or omissions and raise errors early.\n\nThis said, it is also possible to express a DSL in terms of unstructured, or plain atom tags, when\nmore informal definitions might be acceptable:\n\n```elixir\ndefmodule MyApp.Html.Dsl do\n  use Diesel.Dsl,\n    otp_app: :my_app,\n    root: :html,\n    tags: [:head, :body, :meta, :script, ...]\nend\n```","ref":"unstructured-tags.html","title":"Unstructured tags","type":"extras"},{"doc":"# Kernel conflicts\n\nDepending on how you define your DSL, you might get compiler errors in the form `function /Y\nimported from both (YOUR DSL) and Kernel, call is ambiguous`.\n\nYou can stop importing the one from Kernel via the `:overrides` key, eg:\n\n```elixir\ndefmodule MyApp.Html do\n  use Diesel,\n    otp_app: :my_app,\n    dsl: MyApp.Html.Dsl,\n    overrides: [div: 1]\n```","ref":"kernel-conflicts.html","title":"Kernel conflicts","type":"extras"},{"doc":"# Debug\n\nYou can turn debug on, in order to see the code being generated by Diesel at compile time.\n\n```elixir\ndefmodule MyApp.MyFsm do\n  use Diesel, debug: true\n  ...\nend\n```","ref":"debug.html","title":"Debug","type":"extras"}]}